/** @file scdb.h
 * Abstraction layer over the SCdb database.
 */

#ifndef SCDB_H_
#define SCDB_H_

#include <stdio.h>

#ifdef _WIN32
	#include <mysql/my_global.h>
	#warning "using mysql/my_global.h"
#endif /* _WIN32 */

#ifndef ulong
	typedef unsigned long ulong;
#endif

#include <mysql/mysql.h>
#include <string.h>

#include "wlcstdc.h"
#include "iniparser.h"

#include "scdata.h"

/** Stores the resoults of a query */
#define SCDB_RES MYSQL_RES
/** Stores a row from the results */
#define SCDB_ROW MYSQL_ROW

#define SCdb_fetch_row mysql_fetch_row
#define SCdb_free_result mysql_free_result

/** 
 * Parameters needed to connect to the database taken
 * from an ini file.
 */
struct SCdb_cfg
{
	dictionary *d;	/**< dictionare from the ini file */
	char	*host,	/**< host adres or name */
		*user,	/**< user name */
		*passwd, /**< users password */
		*db_name; /**< database name */
};

/**
 * Allocates a SCdb_cfg structure with entries from an ini file.
 * @param ini_path path of the ini file
 * @return a pointer to the allocated SCdb_cfg structure if
 * successful; NULL if couldn't open the file or if the file
 * didn't have the required entries
 */
struct SCdb_cfg* make_SCdb_cfg(const char* ini_path);

/**
 * Initializes a database connection with parameters from cfg.
 * @param cfg database parameters
 * @return 0 if successful else -1
 */
int SCdb_init(const struct SCdb_cfg *cfg);

/**
 * Frees the database connection.
 */
void SCdb_free();

/**
 * Closes the connection and establishes a new one with
 * the parameters from cfg.
 * @param cfg new connection parameters
 * @return 0 if successful else -1
 */
int SCdb_reset_connection(const struct SCdb_cfg *cfg);

/** Returns the results from last query */
SCDB_RES* SCdb_store_result();

/**
 * Execute query from query.
 * @param query a string containing the query
 * @return 0 if successful else -1
 */
int SCdb_query(const char* query);

/** @name stored_proc SCdb stored procedures
 * Functions operating on SCdb stored procedures
 * This is a set of stored procedures predefined in SCdb. See
 * the "Selection Check Developers Manual" for a description
 * of there parameters and about the results they give. All
 * of these functions return 0 if the query was successful or
 * -1 if otherwise.
 */
/** @{ */

/** 
 * Inserts worklog date.
 * See "Selection Check Developer Manual for more
 * information.
 */
int SCdb_insert_wl( const char* date );
/**
 * Inserts SCslaves data record.
 * See "Selection Check Developer Manual for more
 * information.
 */
int SCdb_insert_SCs_data( int over, const char* date,
	const struct SCs_data* data );

/**
 * Gets worklog dates.   
 * See "Selection Check Developer Manual for more
 * information.
 */
int SCdb_get_wls( int year, int month );
/** 
 * Gets planed shift dates.
 * See "Selection Check Developer Manual for more
 * information.
*/
int SCdb_get_planed_shift( int year, int month );
/** 
 * Gets worker names. 
 * See "Selection Check Developer Manual for more
 * information.
 */
int SCdb_get_workers();

/**
 * Inserts worker. 
 * See "Selection Check Developer Manual for more
 * information.
 */
int SCdb_insert_worker( const char* first_name, const char* last_name );
/** 
 * Deletes worker.
 * See "Selection Check Developer Manual for more
 * information.
 */
int SCdb_delete_worker( const char* first_name, const char* last_name );

/** 
 * Deletes worker.
 * See "Selection Check Developer Manual for more
 * information.
 */
 int SCdb_delete_alldata( const char* date );

/**
 * Deletes a shift.
 * See "Selection Check Developer Manual for more
 * information.
 */
int SCdb_delete_shift( const char* date );
/**
 * Inserts a shift.
 * See "Selection Check Developer Manual for more
 * information.
 */
int SCdb_insert_shift( const char* date,
	const char* first_name, const char* last_name,
	int station, int shift );
/**
 * Gets a shift.
 * See "Selection Check Developer Manual for more
 * information.
 */
int SCdb_get_shift(const char* date);

/**
 * Gets work done by one worker in day 'date'.
 * See "Selection Check Developer Manual for more
 * information.
 */
int SCdb_get_work( const char* first_name, const char* last_name,
	const char* date );
/**
 * Gets inputs from a SCslave generated in the specified
 * time.
 * See "Selection Check Developer Manual for more
 * information.
 */
int SCdb_get_input( const char* date,
	const char* tfrom, const char* tto, int ws );

/** @} */

#endif /* SCDB_H_ */

