/** @file scdata.h
 * Functions and structures used to manipulate data in
 * the whole SC system.
 */

/** \mainpage Selection Check Librarys
 * This are the core libraries used by programs in the Selection
 * Check system. It is compiled in to three shared libraries:
 * - libscwork - compiled from sources in ./sc/work/; is used by the work_grp
 * programs,
 * - libscmanage - compiled from sources in ./sc/manage/; is used by the
 * manage_grp programs,
 * - libscanalis - compiled from sources in ./sc/analis/; is used by programs
 * that analyse data, mostly in the manage_grp and optionally in the work_grp.
 *
 * Source files placed directly in ./sc/ are used by all this libraries.
 *
 * This documentation is generated in to a HTML and PDF version.
 * For viewing this documentation on your PC I recommend the
 * HTML version.
 */

#ifndef SCDATA_H_
#define SCDATA_H_

#include <stdio.h>
#include <string.h>
#include <time.h>

#include "md5.h"

#include "wlcstdc.h"

extern const char	*SC_wlset_name,	/**<
					 * the name of a wlset type file
					 */
			*SC_worklogs_dir,	/**<
					 * the name of the directory
					 * that holds worklog files
					 */
			*SC_errorlog_name;	/**<
					 * the name of the error log file
					 */

/** Data record from SCslave */
struct SCs_data
{
	struct tms time; /**< time in which this record was generated */
	unsigned int code; /**< SCslave code */
	unsigned int input; /**< SCslave registers state */ 
};

/** Error record used in errorlog file */
struct SCr_errno
{
	time_t time; /**< date and time when error occurred */
	int type; /**< error type */
	int code; /**< error code */
};


/** Error types codes returned and logged in errorlog file
 * by the SCrunner program. See the "Selection Check
 * Developers Manual" for more detail.
 */
enum SCr_error_type
{
	SCr_err_startup = 0, 	/**< initialization error */
	SCr_err_nocheck = 1,	/**<
				 * raised when the maximum number of
				 * unsuccessful communication tray's
				 * with SCslave module have occurred
				 */
	SCr_err_file_format = 2,/**< wrong file format */
	SCr_err_sys_disk = 3,
	SCr_err_transfer = 4, 	/**< transfer error */
	SCr_err_unloged = 5, 	/**<
				 * multiple unsuccessful data logs
				 */
	SCr_err_types = 6 	/**< number of error types */
};

/**
 * The number of maximum unsuccessful communication tray's
 * with SCslave module
 */
#define SCR_MAX_NOCHECK 5

/** Transfer type error codes */
enum SCt_error_type
{
	SCt_err_argc = 1,	/**< wrong number of
				 * arguments
				 */
	SCt_err_no_twlset = 3, 	/**<
				 * couldn't create a wlset file
				 * in the transfer directory
				 */
	SCt_err_no_owlset = 5, 	/**<
				 * couldn't open the old wlset file
				 */
	SCt_err_no_wlset = 7,	/**<
				 * couldn't open the new wlset file
				 */
	SCt_err_no_terr = 11,	/**<
				 * couldn't create a terror file in
				 * the transfer directory
				 */
	SCt_err_pre_sh = 13,	/**< pre-trans.sh error */
	SCt_err_post_sh = 15,	/**< post-trans.sh error */
	SCt_err_over = 17	/**<
				 * tried to override an old transfer
				 */
};

/**
 * Converts code of an error type to its name.
 * @param type error type from SCr_error_type
 * @return a pointer to a string with the name of
 * the error type
 */
const char* SCr_err_to_str(int type);

/** The length of a string with time in format HH:MM:SS */
#define TIMELEN 9

/**
 * Generates a string with time 'timer' in format HH:MM:SS.
 * @param give generated string
 * @param timer source time
 */
void scode_time(char* give, time_t timer);

/** The length of a string with a date in ISO format */
#define DATELEN 11

/**
 * Generates a string with the date from 'timer' in ISO format.
 * @param give generated string
 * @param timer source date
 */
void scode_date(char* give, time_t timer);

/**
 * Writes an error to the errorlog file
 * @param dst destination file
 * @param err the error
 */
void SCr_err_fwrite(FILE* dst, const struct SCr_errno* err);

/**
 * Reads an error from the error log file
 * @param src source file
 * @param err the error
 */
void SCr_err_fread(FILE* src, struct SCr_errno* err);

/**
 * Writes a data record from SCslave to a worklog file.
 * @param dst destination file
 * @param data data record from SCslave
 * @return 0 if successful else -1
 */
int SCs_data_fwrite(FILE* dst, const struct SCs_data* data);

/**
 * Reades a SCslave data record from worklog file.
 * @param src source file
 * @param data SCslave data record
 * @return 0 if successful else -1
 */
int SCs_data_fread(FILE* src, struct SCs_data* data);

/** The lenght of a string with the MD5 checksum */
#define MD5LEN 16

/**
 * Generates a MD5 checksum for a file. This is a modification of a function
 * from md5sum source.
 * @param fp file for witch the checksum will be generated
 * @param digest byte array in witch the checksume will be stored, must
 * be at least MD5LEN big
 * @return 0 if successful else -1
 */
int md5file(FILE *fp, unsigned char *digest);

#endif /* SCDATA_H_ */

