/*
 * Driver program for the SCslave modules.
 */

#include <stdio.h>
#include <string.h>
#include <time.h>

#include <unistd.h>
#include <sys/wait.h>
#include <sys/timeb.h>
#include <signal.h>

#include "iniparser.h"

#include "work/scrunner_msg.h"
#include "work/schard.h"
#include "scdata.h"
#include "scdata_l.h"

#include "wlcstdc.h"

const char* vr = "1.01";

char strdate[DATELEN];

int	HError = 0, /**< indicates if an critical error occured */
	LError = 0, /**< indicates if an lite error occured */
	SCrunner_off = 0,
	req_sysoff = 0, /**< indicates if system off has been requested */ 
	req_trans = 0; /**< indicates if data transfer has
			* been requested
			*/

/** Programs settings from scrunner.ini file */
struct SCr_cfg
{
	unsigned long	cycle_delay_microsec;
	unsigned	cycle_delay_sec;
	int	SCs_offtest,
		*proc_log_ptr,
		SCmc_on,
		SCs_numb;
	char trans_dir[100];
	int	workstation_codes[SCS_NUMB_MAX];
	char	workstation_paths[SCS_NUMB_MAX][100];
} cfg;

void mark_error(int He, int type, int code, const char* strdate);
void term_hand(int s);
void usr1_hand(int s);
void program_init();
void user_handler();

int main()
{
	int	mi, check = 0, loged = 0,
		nocheck[SCS_NUMB_MAX] = { 0 };
	time_t time_buf;
	struct SCs_data SCs_buf;

	program_init();
	fflush(stdout);

	while(!HError && !SCrunner_off)
	{
		sleep(cfg.cycle_delay_sec);
		usleep(cfg.cycle_delay_microsec);

		for(mi = 0; mi < cfg.SCs_numb; mi += 1)
		{
			if(cfg.workstation_codes[mi] == 0)
				continue;

			check = SCs_data_cycle(
				cfg.workstation_codes[mi] );
			SCs_buf.code = cfg.workstation_codes[mi];
			time_buf = time(NULL);
			tm_to_stm(&(SCs_buf.time), localtime(&time_buf));
			if(check >= 0)
			{
				SCs_buf.input = check;
				nocheck[cfg.workstation_codes[mi]] = 0;
				loged = SCr_log_data(&SCs_buf, strdate);
				if(SCr_not_loged > SCr_max_not_loged)
				{
					mark_error(1, SCr_err_unloged,
						SCr_not_loged, strdate);
					break;
				}
			}
			else if( (check == -2) && cfg.SCs_offtest )
			{
				nocheck[cfg.workstation_codes[mi]]
					+= 1;
				if( nocheck[cfg.workstation_codes[mi]]
					> SCR_MAX_NOCHECK )
				{
					mark_error( 0, SCr_err_nocheck,
					cfg.workstation_codes[mi],
					strdate );
				}
			}
		}
		user_handler();
	}
	
	if(HError)
	{
		
		puts("Critical error occured!");
		while(!req_sysoff)
		{
			system("echo -n -e \"\a\""); /* or \007*/
			sleep(1);
		}
	}
	
	fputs(SCR_END_MSG, stdout);
	system("sh exit.sh");
	return HError;
}

void mark_error(int He, int type, int code, const char* strdate)
{
	if(He)
		HError = 1;
	else
	{
		LError = 1;
		system("echo -n -e \"\a\""); /* or \007*/
	}
	
	SCr_log_error(type, code, strdate);
}

void term_hand(int s)
{
	req_sysoff = 1;
}

void usr1_hand(int s)
{
	req_trans = 1;
}

void SCr_cfg_init(struct SCr_cfg* cfg)
{
	cfg->proc_log_ptr = &SCr_proc_log_on;
}

int read_settings(dictionary* settings)
{
	char 	*str_ptr = NULL,
		strbuf[20];
	int i, bits;
	
	/* main */
	cfg.SCs_offtest = iniparser_getboolean(settings,
		"main:scs_offtest", -1);
	if(cfg.SCs_offtest == -1)
		return -1;
	*(cfg.proc_log_ptr) = iniparser_getboolean(settings,
		"main:proclog", -1);
	if(*(cfg.proc_log_ptr) == -1)
		return -1;
	str_ptr = iniparser_getstr(settings, "main:trans_dir");
	if(!str_ptr)
		return -1;
	else
		strcpy(cfg.trans_dir, str_ptr);
	
	/* delays */	
	cfg.cycle_delay_sec = iniparser_getint(settings,
		"delays:cycle_sec", 1);
	cfg.cycle_delay_microsec = iniparser_getint(settings,
		"delays:cycle_microsec", 0);
		
	/*workstations*/
	cfg.SCs_numb = 0;
	for(i = 1; i <= SCS_NUMB_MAX; i += 1)
	{
		sprintf(strbuf, "workstations:%u", i);
		str_ptr = iniparser_getstr(settings, strbuf);
		if(str_ptr)
		{
			cfg.workstation_codes[cfg.SCs_numb] = i;
			strcpy( cfg.workstation_paths[cfg.SCs_numb],
				str_ptr );
			cfg.SCs_numb += 1;
		}
	}

	return 0;
}

void program_init()
{
	int i, setch, isch;
	const char *settings_name = "scrunner.ini";
	FILE *wlset = fopen(SC_wlset_name,"a");
	dictionary *settings = iniparser_load(settings_name);
	int scand = 0;
	SCr_cfg_init(&cfg);
	
	scode_date(strdate, time(0));

	printf(SCR_INIT_MSG, vr, getpid());
	putchar('\n');	

	printf("Reading settings:");

	if(!settings)
	{
		printf( "\tERROR-couldn't open or wrong format of file %s\n",
			settings_name );
		mark_error( 1, SCr_err_startup, 2, strdate );
		if(wlset)
			fclose(wlset);
		return;
	}
	if(!wlset)
	{
		printf("\tERROR-couldn't open file %s\n", SC_wlset_name);
		mark_error(1, SCr_err_startup, 4, strdate);

		iniparser_freedict(settings);
		return;
	}
	
	printf("\n\t- %s", settings_name);
	
	if(read_settings(settings) == -1)
	{
			mark_error(1, SCr_err_startup, 6, strdate);
			puts("\tERROR-lack of required values");
			iniparser_freedict(settings);
			fclose(wlset);
			return;
	}

	puts("\tOK");
	fflush(stdout);
	scand = 0;
	iniparser_freedict(settings);
	
	printf("\t- %s.wl", strdate);
	switch(SCr_new_wl(strdate))
	{
		case SCr_wl_state_old : break;
		case SCr_wl_state_new : fprintf(wlset, "%s\n", strdate); break;
		case SCr_wl_state_err :
		{
			printf( "\tERROR-couldn't open or create file %s.wl\n",
				strdate );
			mark_error( 1, SCr_err_startup, 10, strdate );
			fclose(wlset);
			return;
		} break;
	}
	puts("\tOK");
	
	printf("\t- %s\tOK", SC_wlset_name);
	fclose(wlset);
	
	printf("\nInitializing hardware:");
	if(SC_hard_init() == 0)
		puts("\tOK");
	else
	{
		puts("\tERROR\n");
		mark_error(1, SCr_err_startup, 12, strdate);
		return;
	}
	fflush(stdout);

	printf("\nSetting signel handlers:");
	if(	(signal(SIGTERM, term_hand) == SIG_ERR) ||
		(signal(SIGQUIT, term_hand) == SIG_ERR) ||
		(signal(SIGUSR1, usr1_hand) == SIG_ERR))
	{
		puts("\tERROR-couldn't set all signal handlers");
		mark_error(1, SCr_err_startup, 14, strdate);
		return;
	}
	else
		puts("\tOK");
	
	isch = 1;
	printf("Initializing SCslave modules:");
	for(i = 0; i < cfg.SCs_numb; i += 1)
	{
		if( SCs_init( cfg.workstation_paths[i],
			cfg.workstation_codes[i] ) != 0 )
		{
			isch = 0;
			printf( "\n\t-ERROR - module %i id not working "
				"properlyy", cfg.workstation_codes[i] );
			if(cfg.SCs_offtest)
				cfg.workstation_codes[i] = 0;
		}
	}
	if(isch)
		puts("\tOK");
	else
		putchar('\n');
	
	fputs(SCR_PROCLOG_START_MSG, stdout);
}

void user_handler()
{
	static int tpid = 0;
	int tstatus;
	char str_buf[100];

	if(tpid != 0)
	{
		waitpid(tpid, &tstatus, WNOHANG);
		if(WIFEXITED(tstatus))
		{
			tpid = 0;
			
			if(WEXITSTATUS(tstatus) != 0)
				mark_error(0, SCr_err_transfer,
					WEXITSTATUS(tstatus), strdate);
			
			strcpy(str_buf, cfg.trans_dir);
			rename(SC_errorlog_name,
				strcat(str_buf, SC_errorlog_name));
			SCr_log_proc(SCr_proc_user, SCr_user_trans_end,
				0, strdate);
		}
	}
	
	if(req_sysoff)
	{
		SCrunner_off = 1;
		SCr_log_proc(SCr_proc_user, SCr_user_sys_off, 0, strdate);
	}
	if(req_trans)
	{
		req_trans = 0;
		SCr_log_proc(SCr_proc_user, SCr_user_trans_start, 0, strdate);
		switch(tpid = fork())
		{
			case -1 : mark_error(0, SCr_err_transfer, 21, strdate);
			case 0 :
			{
				execl("./scwl2dir", "sctwl2dir", cfg.trans_dir, (char*)0);
			} break;
		}
	}
}

